import math

from torch.nn.init import _calculate_fan_in_and_fan_out, _no_grad_normal_, _no_grad_uniform_


def early_stop(val_acc_history, tasks, early_stop_step_single,
                early_stop_step_multi, required_progress):
  """
  Stop the training if there is no non-trivial progress in k steps
  @param val_acc_history: a list contains all the historical validation acc
  @param required_progress: the next acc should be higher than the previous by
      at least required_progress amount to be non-trivial
  @param t: number of training steps
  @return: a boolean indicates if the model should early stop
  """
  # TODO: add your code here
  if len(tasks) == 1:
    t = early_stop_step_single
  else:
    t = early_stop_step_multi

  if len(val_acc_history) > t:
    if val_acc_history[-1] - val_acc_history[-1 - t] < required_progress:
      return True
  return False


def xavier_normal_small_init_(tensor, gain=1.):
  # type: (Tensor, float) -> Tensor
  fan_in, fan_out = _calculate_fan_in_and_fan_out(tensor)
  std = gain * math.sqrt(2.0 / float(fan_in + 4 * fan_out))

  return _no_grad_normal_(tensor, 0., std)


def xavier_uniform_small_init_(tensor, gain=1.):
  # type: (Tensor, float) -> Tensor
  fan_in, fan_out = _calculate_fan_in_and_fan_out(tensor)
  std = gain * math.sqrt(2.0 / float(fan_in + 4 * fan_out))
  a = math.sqrt(3.0) * std  # Calculate uniform bounds from standard deviation

  return _no_grad_uniform_(tensor, -a, a)
